"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = buildCss;

var _path = _interopRequireDefault(require("path"));

var _util = _interopRequireDefault(require("util"));

var _gulp = require("gulp");

var _gulpIf = _interopRequireDefault(require("gulp-if"));

var _through = _interopRequireDefault(require("through2"));

var _vinyl = _interopRequireDefault(require("vinyl"));

var _sass = _interopRequireDefault(require("sass"));

var _fibers = _interopRequireDefault(require("fibers"));

var _nodeSassMagicImporter = _interopRequireDefault(require("node-sass-magic-importer"));

var _gulpPostcss = _interopRequireDefault(require("gulp-postcss"));

var _autoprefixer = _interopRequireDefault(require("autoprefixer"));

var _cssMqpacker = _interopRequireDefault(require("css-mqpacker"));

var _postcssUncss = _interopRequireDefault(require("postcss-uncss"));

var _postcssCsso = _interopRequireDefault(require("postcss-csso"));

var _postcssUrl = _interopRequireDefault(require("postcss-url"));

var _gulpHeader = _interopRequireDefault(require("gulp-header"));

var _gulpRename = _interopRequireDefault(require("gulp-rename"));

var _gulpGzip = _interopRequireDefault(require("gulp-gzip"));

var _fancyLog = _interopRequireDefault(require("fancy-log"));

var _gulpUtilErrorHandler = _interopRequireDefault(require("@hidoo/gulp-util-error-handler"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let pkg = {}; // try to load package.json that on current working directory

try {
  // eslint-disable-next-line node/global-require, import/no-dynamic-require
  pkg = require(_path.default.resolve(process.cwd(), 'package.json'));
} catch (error) {
  _fancyLog.default.error('Failed to load package.json.');
}
/**
 * default custom sass functions
 *
 * @type {Object}
 */


const DEFAULT_FUNCTIONS = {
  /**
   * add "env" function that get value from process.env
   *
   * @param {String} key key
   * @param {Function} done callback function when call asynchronously
   * @return {void|sass.compiler.types.String}
   */
  'env($key)': (key, done) => {
    const name = key.getValue(),
          result = new _sass.default.types.String(process.env[name] || '' // eslint-disable-line node/no-process-env
    );

    if (typeof done === 'function') {
      return done(result);
    }

    return result;
  }
};
/**
 * task default options.
 *
 * @type {Object}
 */

const DEFAULT_OPTIONS = {
  name: 'build:css',
  src: null,
  dest: null,
  filename: 'main.css',
  suffix: '.min',
  targets: ['> 0.5% in JP', 'ie >= 10', 'android >= 4.4'],
  banner: '',
  sassOptions: {
    outputStyle: 'expanded'
  },
  url: null,
  urlOptions: {},
  uncssTargets: [],
  uncssIgnore: [],
  additionalProcess: null,
  compress: false
};
/**
 * promisifed sass.render
 *
 * @param {Object} options options of sass.render
 * @return {Promise<Object>}
 */

const render = _util.default.promisify(_sass.default.render).bind(_sass.default);
/**
 * get post css processes
 *
 * @param {DEFAULT_OPTIONS} options - options
 * @return {Array<Function>}
 */


function getProcesses(options = {}) {
  const processes = [(0, _autoprefixer.default)({
    overrideBrowserslist: options.browsers || options.targets || []
  }), _cssMqpacker.default]; // add post css process by postcss-url

  if (typeof options.url === 'string' && ['inline', 'copy', 'rebase'].includes(options.url)) {
    processes.push((0, _postcssUrl.default)({
      basePath: _path.default.dirname(options.src),
      ...options.urlOptions,
      url: options.url
    }));
  } // add post css process if options.uncssTargets is valid


  if (Array.isArray(options.uncssTargets) && Boolean(options.uncssTargets.length)) {
    processes.push((0, _postcssUncss.default)({
      html: options.uncssTargets,
      ignore: Array.isArray(options.uncssIgnore) ? options.uncssIgnore : []
    }));
  } // add post css process if options.additionalProcess is valid


  if (typeof options.additionalProcess === 'function') {
    processes.push(options.additionalProcess);
  }

  return processes;
}
/**
 * return css build task by sass
 *
 * @param  {Object} options - options
 * @param  {String} [options.name='build:css'] - task name (use as displayName)
 * @param  {String} options.src - source path
 * @param  {String} options.dest - destination path
 * @param  {String} [options.filename='main.css'] - destination filename
 * @param  {String} [options.suffix='.min'] - suffix when compressed
 * @param  {Array<String>} [options.targets] - target browsers.
 *   see: {@link http://browserl.ist/?q=%3E+0.5%25+in+JP%2C+ie%3E%3D+10%2C+android+%3E%3D+4.4 default target browsers}
 * @param  {Array<String>} [options.browsers] - alias of options.targets.
 * @param  {String} [options.banner=''] - license comments
 * @param  {Object} [options.sassOptions={outputStyle: 'expanded'}] - sass options.
 *   see: {@link https://sass-lang.com/documentation/js-api#options sass options}
 * @param  {String} [options.url=null] - type of processing of url() (one of [inline|copy|rebase])
 *   see: {@link https://www.npmjs.com/package/postcss-url}
 * @param  {Object} [options.urlOptions={}] - options of processing of url()
 *   see: {@link https://www.npmjs.com/package/postcss-url#options-combinations}
 * @param  {Array<String>} [options.uncssTargets=[]] - array of html file path that target of uncss process
 * @param  {Array<String>} [options.uncssIgnore=[]] - array of selector that should not removed
 * @param  {Function<PostCSSRoot>} [options.additionalProcess=null] - additional PostCss process
 * @param  {Boolean} [options.compress=false] - compress file or not
 * @return {Function<Stream>}
 *
 * @example
 * import {task} from 'gulp';
 * import buildCss from '@hidoo/gulp-task-build-css-sass';
 *
 * task('css', buildCss({
 *   name: 'css:main',
 *   src: '/path/to/scss/main.scss',
 *   dest: '/path/to/dest',
 *   filename: 'styles.css',
 *   suffix: '.hoge',
 *   targets: ['> 0.1% in JP'],
 *   banner: '/*! copyright <%= pkg.author %> * /\n',
 *   sassOptions: {outputStyle: 'nested'},
 *   url: 'inline',
 *   urlOptions: {basePath: path.resolve(process.cwd(), 'src/images')},
 *   uncssTargets: ['/path/to/html/target.html'],
 *   uncssIgnore: ['.ignore-selector'],
 *   additionalProcess: (root) => root,
 *   compress: true
 * }));
 */


function buildCss(options = {}) {
  const opts = { ...DEFAULT_OPTIONS,
    ...options
  }; // define task

  const task = () => {
    const {
      suffix,
      compress
    } = opts;

    const stream = _through.default.obj();

    const filename = opts.filename || _path.default.basename(opts.src);

    const sassOptions = opts.sassOptions || {};
    const processes = getProcesses(opts);
    const compiler = render({
      file: opts.src,
      ...sassOptions,
      fiber: _fibers.default,
      importer: (0, _nodeSassMagicImporter.default)(),
      functions: { ...DEFAULT_FUNCTIONS,
        ...sassOptions.functions
      }
    });
    compiler.then(({
      css
    }) => {
      // add code to stream as vinyl file
      stream.push(new _vinyl.default({
        path: filename,
        contents: Buffer.from(css)
      })); // add null that indicates write completion

      stream.push(null);
    }).catch(error => {
      (0, _gulpUtilErrorHandler.default)(error);
      stream.emit('end');
    });
    return stream.pipe((0, _gulpPostcss.default)(processes)).pipe((0, _gulpHeader.default)(opts.banner, {
      pkg
    })).pipe((0, _gulp.dest)(opts.dest)).pipe((0, _gulpIf.default)(compress, (0, _gulpRename.default)({
      suffix
    }))).pipe((0, _gulpIf.default)(compress, (0, _gulpPostcss.default)([(0, _postcssCsso.default)({
      restructure: false
    })]))).pipe((0, _gulpIf.default)(compress, (0, _gulp.dest)(opts.dest))).pipe((0, _gulpIf.default)(compress, (0, _gulpGzip.default)({
      append: true
    }))).pipe((0, _gulpIf.default)(compress, (0, _gulp.dest)(opts.dest)));
  }; // add displayName (used as task name for gulp)


  if (typeof opts.name === 'string' && opts.name !== '') {
    task.displayName = opts.name;
  }

  return task;
}